@@ -24,11 +24,11 @@
 #include <poll.h>
 #include <zlib.h>
 #include <libgen.h>
-#include <json-glib/json-glib.h>
 #include "connector.h"
 #include "utils.h"
 #include "sample.h"
 #include "package.h"
+#include "../config.h"
 
 #define KB 1024
 #define BUFF_SIZE (4 * KB)
@@ -38,19 +38,6 @@
 #define POLL_TIMEOUT 20
 #define MAX_ZIP_SIZE (128 * 1024 * 1024)
 
-#define AFMK1_ID 0x04
-#define AKEYS_ID 0x06
-#define ARMK1_ID 0x08
-#define AHMK1_ID 0x0a
-#define DTAKT_ID 0x0c
-#define AFMK2_ID 0x0e
-#define ARMK2_ID 0x10
-#define DTONE_ID 0x14
-#define AHMK2_ID 0x16
-#define DKEYS_ID 0x1c
-#define MOD_S_ID 0x19
-#define MOD_C_ID 0x1b
-
 #define FS_DATA_PRJ_PREFIX "/projects"
 #define FS_DATA_SND_PREFIX "/soundbanks"
 #define FS_SAMPLES_START_POS 5
@@ -252,109 +239,6 @@ static const guint8 OS_UPGRADE_WRITE_RESPONSE[] =
 
 static const gchar *FS_TYPE_NAMES[] = { "+Drive", "RAM" };
 
-static const struct connector_device_desc ANALOG_FOUR_DESC = {
-  .name = "Analog Four",
-  .alias = "af",
-  .id = AFMK1_ID,
-  .fss = FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = 0
-};
-
-static const struct connector_device_desc ANALOG_KEYS_DESC = {
-  .name = "Analog Keys",
-  .alias = "ak",
-  .id = AKEYS_ID,
-  .fss = FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = 0
-};
-
-static const struct connector_device_desc ANALOG_RYTM_DESC = {
-  .name = "Analog Rytm",
-  .alias = "ar",
-  .id = ARMK1_ID,
-  .fss = FS_SAMPLES | FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = STORAGE_PLUS_DRIVE | STORAGE_RAM
-};
-
-static const struct connector_device_desc ANALOG_HEAT_DESC = {
-  .name = "Analog Heat",
-  .alias = "ah",
-  .id = AHMK1_ID,
-  .fss = FS_DATA_ALL | FS_DATA_SND,
-  .storages = 0
-};
-
-static const struct connector_device_desc DIGITAKT_DESC = {
-  .name = "Digitakt",
-  .alias = "dt",
-  .id = DTAKT_ID,
-  .fss = FS_SAMPLES | FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = STORAGE_PLUS_DRIVE | STORAGE_RAM
-};
-
-static const struct connector_device_desc ANALOG_FOUR_MKII_DESC = {
-  .name = "Analog Four MKII",
-  .alias = "af",
-  .id = AFMK2_ID,
-  .fss = FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = 0
-};
-
-static const struct connector_device_desc ANALOG_RYTM_MKII_DESC = {
-  .name = "Analog Rytm MKII",
-  .alias = "ar",
-  .id = ARMK2_ID,
-  .fss = FS_SAMPLES | FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = STORAGE_PLUS_DRIVE | STORAGE_RAM
-};
-
-static const struct connector_device_desc DIGITONE_DESC = {
-  .name = "Digitone",
-  .alias = "dn",
-  .id = DTONE_ID,
-  .fss = FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = 0
-};
-
-static const struct connector_device_desc ANALOG_HEAT_MKII_DESC = {
-  .name = "Analog Heat MKII",
-  .alias = "ah",
-  .id = AHMK2_ID,
-  .fss = FS_DATA_ALL | FS_DATA_SND,
-  .storages = 0
-};
-
-static const struct connector_device_desc DIGITONE_KEYS_DESC = {
-  .name = "Digitone Keys",
-  .alias = "dn",
-  .id = DKEYS_ID,
-  .fss = FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = 0
-};
-
-static const struct connector_device_desc MODEL_SAMPLES_DESC = {
-  .name = "Model:Samples",
-  .alias = "ms",
-  .id = MOD_S_ID,
-  .fss = FS_SAMPLES | FS_DATA_ALL | FS_DATA_PRJ | FS_DATA_SND,
-  .storages = STORAGE_PLUS_DRIVE | STORAGE_RAM
-};
-
-static const struct connector_device_desc MODEL_CYCLES_DESC = {
-  .name = "Model:Cycles",
-  .alias = "mc",
-  .id = MOD_C_ID,
-  .fss = FS_RAW_ALL | FS_RAW_PRESETS | FS_DATA_ALL | FS_DATA_PRJ,
-  .storages = STORAGE_PLUS_DRIVE
-};
-
-static const struct connector_device_desc *CONNECTOR_DEVICE_DESCS[] = {
-  &ANALOG_FOUR_DESC, &ANALOG_KEYS_DESC, &ANALOG_RYTM_DESC, &ANALOG_HEAT_DESC,
-  &DIGITAKT_DESC, &ANALOG_FOUR_MKII_DESC, &ANALOG_RYTM_MKII_DESC,
-  &DIGITONE_DESC, &ANALOG_HEAT_MKII_DESC, &DIGITONE_KEYS_DESC,
-  &MODEL_SAMPLES_DESC, &MODEL_CYCLES_DESC
-};
-
 static const struct fs_operations FS_SAMPLES_OPERATIONS = {
   .fs = FS_SAMPLES,
   .readdir = connector_read_samples_dir,
@@ -2362,7 +2246,7 @@ connector_destroy (struct connector *connector)
     {
       free (connector->device_name);
       free (connector->fw_version);
-      free (connector->alias);
+      free (connector->overbridge_name);
       connector->device_name = NULL;
     }
 
@@ -2383,6 +2267,19 @@ connector_destroy (struct connector *connector)
       g_hash_table_destroy (connector->dir_cache);
       connector->dir_cache = NULL;
     }
+
+  if (connector->device_desc.name)
+    {
+      g_free (connector->device_desc.name);
+      connector->device_desc.name = NULL;
+    }
+
+  if (connector->device_desc.alias)
+    {
+      g_free (connector->device_desc.alias);
+      connector->device_desc.alias = NULL;
+    }
+
 }
 
 gint
@@ -2441,27 +2338,12 @@ connector_get_storage_stats_percent (struct connector_storage_stats *statfs)
   return (statfs->bsize - statfs->bfree) * 100.0 / statfs->bsize;
 }
 
-static const struct connector_device_desc *
-connector_get_device_desc (guint8 id)
-{
-  guint total =
-    sizeof (CONNECTOR_DEVICE_DESCS) / sizeof (struct connector_device_desc *);
-  guint i;
-
-  for (i = 0; i < total; i++)
-    {
-      if (id == CONNECTOR_DEVICE_DESCS[i]->id)
-	{
-	  return CONNECTOR_DEVICE_DESCS[i];
-	}
-    }
-  return NULL;
-}
-
 gint
-connector_init (struct connector *connector, gint card)
+connector_init (struct connector *connector, gint card,
+		const char *device_filename)
 {
-  int err;
+  gint err;
+  guint8 id;
   GByteArray *tx_msg;
   GByteArray *rx_msg;
   snd_rawmidi_params_t *params;
@@ -2474,6 +2356,8 @@ connector_init (struct connector *connector, gint card)
   connector->rx_len = 0;
   connector->pfds = NULL;
   connector->dir_cache = NULL;
+  connector->device_desc.name = NULL;
+  connector->device_desc.alias = NULL;
   if (card < 0)
     {
       debug_print (1, "Invalid card\n");
@@ -2562,10 +2446,11 @@ connector_init (struct connector *connector, gint card)
       err = -EIO;
       goto cleanup_params;
     }
-  connector->alias = strdup ((gchar *) & rx_msg->data[7 + rx_msg->data[6]]);
-  connector->device_desc = connector_get_device_desc (rx_msg->data[5]);
+  connector->overbridge_name =
+    strdup ((gchar *) & rx_msg->data[7 + rx_msg->data[6]]);
+  id = rx_msg->data[5];
   free_msg (rx_msg);
-  if (!connector->device_desc)
+  if (load_device_desc (&connector->device_desc, id, device_filename))
     {
       err = -ENODEV;
       goto cleanup_params;
@@ -2596,8 +2481,8 @@ connector_init (struct connector *connector, gint card)
     }
 
   snprintf (connector->device_name, LABEL_MAX, "%s %s (%s)",
-	    connector->device_desc->name,
-	    connector->fw_version, connector->alias);
+	    connector->device_desc.name,
+	    connector->fw_version, connector->overbridge_name);
   debug_print (1, "Connected to %s\n", connector->device_name);
   err = 0;
 
@@ -3419,7 +3304,7 @@ connector_download_pkg (const gchar * path, GByteArray * output,
     }
 
   if (package_begin
-      (&pkg, pkg_name, connector->fw_version, connector->device_desc, type))
+      (&pkg, pkg_name, connector->fw_version, &connector->device_desc, type))
     {
       g_free (pkg_name);
       return -1;
@@ -3620,7 +3505,7 @@ connector_get_download_path (struct connector *connector,
   name = connector_get_download_name (connector, remote_iter, ops, src_fpath);
   if (name)
     {
-      dl_ext = connector_get_full_ext (connector->device_desc, ops);
+      dl_ext = connector_get_full_ext (&connector->device_desc, ops);
       filename = malloc (PATH_MAX);
       snprintf (filename, PATH_MAX, "%s.%s%s", name, dl_ext, md_ext);
       path = chain_path (dst_dir, filename);
@@ -3809,7 +3694,7 @@ connector_upload_pkg (const gchar * path, GByteArray * input,
   struct package pkg;
   struct connector *connector = data;
 
-  ret = package_open (&pkg, input, connector->device_desc);
+  ret = package_open (&pkg, input, &connector->device_desc);
   if (!ret)
     {
       ret = package_send_pkg_resources (&pkg, path, control, connector,
