@@ -28,22 +28,13 @@
 #define SYSEX_TIMEOUT 5000
 #define REST_TIME 50000
 
-struct connector_device_desc
-{
-  const gchar *name;
-  const gchar *alias;
-  guint8 id;
-  guint8 fss;
-  guint8 storages;
-};
-
 struct connector
 {
-  const struct connector_device_desc *device_desc;
+  struct connector_device_desc device_desc;
   snd_rawmidi_t *inputp;
   snd_rawmidi_t *outputp;
   gchar *device_name;
-  gchar *alias;
+  gchar *overbridge_name;
   gchar *fw_version;
   gushort seq;
   GMutex mutex;
@@ -115,7 +106,7 @@ struct connector_storage_stats
 
 const struct fs_operations *connector_get_fs_operations (enum connector_fs);
 
-gint connector_init (struct connector *, gint);
+gint connector_init (struct connector *, gint, const char *);
 
 void connector_destroy (struct connector *);
 
