@@ -36,6 +36,7 @@
 
 static struct connector connector;
 static struct job_control control;
+static const char *devices_filename;
 
 typedef void (*print_item) (struct item_iterator *);
 
@@ -109,11 +110,11 @@ static gint
 cli_connect (const char *device_path, enum connector_fs fs)
 {
   gint card = atoi (device_path);
-  gint ret = connector_init (&connector, card);
-  if (!ret && fs && !(connector.device_desc->fss & fs))
+  gint ret = connector_init (&connector, card, devices_filename);
+  if (!ret && fs && !(connector.device_desc.filesystems & fs))
     {
       error_print ("Filesystem not supported for device '%s'\n",
-		   connector.device_desc->name);
+		   connector.device_desc.name);
       return 1;
     }
   return ret;
@@ -270,7 +271,7 @@ cli_info (int argc, char *argv[], int optind)
   comma = "";
   for (int fs = FS_SAMPLES, i = 0; fs <= FS_DATA_SND; fs <<= 1, i++)
     {
-      if (connector.device_desc->fss & fs)
+      if (connector.device_desc.filesystems & fs)
 	{
 	  printf ("%s%s", comma, CLI_FSS[i]);
 	}
@@ -307,7 +308,7 @@ cli_df (int argc, char *argv[], int optind)
       return EXIT_FAILURE;
     }
 
-  if (!connector.device_desc->storages)
+  if (!connector.device_desc.storage)
     {
       return EXIT_FAILURE;
     }
@@ -318,7 +319,7 @@ cli_df (int argc, char *argv[], int optind)
   res = 0;
   for (storage = STORAGE_PLUS_DRIVE; storage <= STORAGE_RAM; storage <<= 1)
     {
-      if (connector.device_desc->storages & storage)
+      if (connector.device_desc.storage & storage)
 	{
 	  res |= connector_get_storage_stats (&connector, storage, &statfs);
 	  if (res)
@@ -602,10 +603,15 @@ main (int argc, char *argv[])
       sigaction (SIGHUP, &action, NULL);
     }
 
-  while ((c = getopt (argc, argv, "v")) != -1)
+  devices_filename = NULL;
+
+  while ((c = getopt (argc, argv, "f:v")) != -1)
     {
       switch (c)
 	{
+	case 'f':
+	  devices_filename = optarg;
+	  break;
 	case 'v':
 	  vflg++;
 	  break;
