@@ -25,7 +25,6 @@
 #include <glib-unix.h>
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
-#define _GNU_SOURCE
 #include <getopt.h>
 #include "connector.h"
 #include "browser.h"
@@ -261,21 +260,60 @@ elektroid_set_file_extensions_for_fs (gchar ** extensions[],
 				      enum connector_fs sel_fs)
 {
   const struct fs_operations *ops;
+  gchar *mp3_ext = NULL;
 
-  if (*extensions != NULL && *extensions != ELEKTROID_AUDIO_LOCAL_EXTS)
+  if (*extensions != NULL)
     {
-      g_free ((*extensions)[0]);
+      gchar **ext = *extensions;
+      while (*ext)
+	{
+	  g_free (*ext);
+	  ext++;
+	}
       g_free (*extensions);
     }
+
   if (sel_fs == FS_SAMPLES)
     {
-      *extensions = ELEKTROID_AUDIO_LOCAL_EXTS;
+      gchar **ext = ELEKTROID_AUDIO_LOCAL_EXTS;
+      int known_ext = 0;
+      while (*ext)
+	{
+	  known_ext++;
+	  ext++;
+	}
+      known_ext++;		//NULL included
+
+      int total_ext = known_ext;
+
+      if (sample_is_mp3_supported ())
+	{
+	  mp3_ext = "mp3";
+	  total_ext++;
+	}
+
+      *extensions = malloc (sizeof (gchar *) * total_ext);
+      ext = ELEKTROID_AUDIO_LOCAL_EXTS;
+      int i = 0;
+      while (*ext)
+	{
+	  (*extensions)[i] = strdup (*ext);
+	  ext++;
+	  i++;
+	}
+      if (mp3_ext)
+	{
+	  (*extensions)[i] = strdup (mp3_ext);
+	  i++;
+	}
+      (*extensions)[i] = NULL;
+
     }
   else
     {
       ops = connector_get_fs_operations (sel_fs);
       *extensions = malloc (sizeof (gchar *) * 2);
-      (*extensions)[0] = connector_get_full_ext (connector.device_desc, ops);
+      (*extensions)[0] = connector_get_full_ext (&connector.device_desc, ops);
       (*extensions)[1] = NULL;
     }
 }
@@ -378,7 +416,7 @@ elektroid_update_statusbar ()
       for (storage = STORAGE_PLUS_DRIVE; storage <= STORAGE_RAM;
 	   storage <<= 1)
 	{
-	  if (connector.device_desc->storages & storage)
+	  if (connector.device_desc.storage & storage)
 	    {
 	      res =
 		connector_get_storage_stats (&connector, storage, &statfs);
@@ -825,7 +863,7 @@ elektroid_show_about (GtkWidget * object, gpointer data)
 }
 
 static void
-elektroid_controls_set_sensitive (gboolean sensitive)
+elektroid_sample_controls_set_sensitive (gboolean sensitive)
 {
   gtk_widget_set_sensitive (local_play_menuitem, sensitive);
   gtk_widget_set_sensitive (play_button, sensitive);
@@ -846,7 +884,7 @@ elektroid_update_ui_on_load (gpointer data)
     {
       if (audio_check (&audio))
 	{
-	  elektroid_controls_set_sensitive (TRUE);
+	  elektroid_sample_controls_set_sensitive (TRUE);
 	}
       if (preferences.autoplay)
 	{
@@ -1303,6 +1341,7 @@ elektroid_local_check_selection (gpointer data)
   GtkTreeIter iter;
   gchar *sample_path;
   GtkTreeModel *model;
+  gboolean audio_controls;
   struct item *item = NULL;
   gint count = browser_get_selected_items_count (&local_browser);
 
@@ -1327,7 +1366,6 @@ elektroid_local_check_selection (gpointer data)
       elektroid_stop_load_thread ();
       audio_reset_sample (&audio);
       gtk_widget_queue_draw (waveform_draw_area);
-      elektroid_controls_set_sensitive (FALSE);
 
       if (item && item->type == ELEKTROID_FILE
 	  && strcmp (item->name, audio.name))
@@ -1336,11 +1374,14 @@ elektroid_local_check_selection (gpointer data)
 	  elektroid_start_load_thread (sample_path);
 	  strcpy (audio.name, item->name);
 	}
-      gtk_widget_set_sensitive (local_open_menuitem, item
-				&& item->type == ELEKTROID_FILE);
+
     }
 
 end:
+  audio_controls = item && item->type == ELEKTROID_FILE && (!remote_browser.fs_ops || remote_browser.fs_ops->fs == FS_SAMPLES);	//1 sample file
+  elektroid_sample_controls_set_sensitive (audio_controls);
+  gtk_widget_set_sensitive (local_open_menuitem, audio_controls);
+
   if (item)
     {
       browser_free_item (item);
@@ -2452,7 +2493,7 @@ elektroid_fill_fs_combo ()
 
   for (int fs = FS_SAMPLES, i = 0; fs <= FS_DATA_SND; fs = fs << 1, i++)
     {
-      if (GUI_FSS & fs && connector.device_desc->fss & fs)
+      if (GUI_FSS & fs && connector.device_desc.filesystems & fs)
 	{
 	  gtk_list_store_insert_with_values (fs_list_store, NULL, -1,
 					     FS_LIST_STORE_ID_FIELD,
@@ -2487,7 +2528,7 @@ elektroid_set_device (GtkWidget * object, gpointer data)
 
       card = g_value_get_uint (&cardv);
 
-      if (connector_init (&connector, card) < 0)
+      if (connector_init (&connector, card, NULL) < 0)
 	{
 	  error_print ("Error while connecting\n");
 	}
@@ -2887,24 +2928,12 @@ elektroid_run (int argc, char *argv[])
   GtkWidget *hostname_label;
   GtkWidget *loop_button;
   GtkWidget *autoplay_switch;
-  char *glade_file = malloc (PATH_MAX);
-  char *css_file = malloc (PATH_MAX);
-  char hostname[LABEL_MAX];
+  gchar *glade_file;
+  gchar *css_file;
+  gchar hostname[LABEL_MAX];
 
-  if (snprintf
-      (glade_file, PATH_MAX, "%s/%s/res/gui.glade", DATADIR,
-       PACKAGE) >= PATH_MAX)
-    {
-      error_print ("Path too long\n");
-      return -1;
-    }
-
-  if (snprintf
-      (css_file, PATH_MAX, "%s/%s/res/gui.css", DATADIR, PACKAGE) >= PATH_MAX)
-    {
-      error_print ("Path too long\n");
-      return -1;
-    }
+  glade_file = chain_path (DATADIR, "gui.glade");
+  css_file = chain_path (DATADIR, "gui.css");
 
   gtk_init (&argc, &argv);
   builder = gtk_builder_new ();
@@ -3260,6 +3289,7 @@ elektroid_run (int argc, char *argv[])
     }
 
   audio_destroy (&audio);
+
   return EXIT_SUCCESS;
 }
 
