@@ -178,7 +178,7 @@ package_add_manifest (struct package *pkg)
       json_builder_add_string_value (builder, pkg->fw_version);
     }
 
-  if (pkg->device_desc->fss & FS_SAMPLES)
+  if (pkg->device_desc->filesystems & FS_SAMPLES)
     {
       for (resource = pkg->resources; resource; resource = resource->next)
 	{
@@ -407,7 +407,6 @@ package_receive_pkg_resources (struct package *pkg,
   if (!reader)
     {
       error_print ("Unable to read from parser. Continuing...");
-      g_object_unref (reader);
       control->parts = 2;
       goto get_payload;
     }
@@ -684,22 +683,26 @@ package_send_pkg_resources (struct package *pkg,
   json_reader_end_element (reader);
   json_reader_end_element (reader);
 
-  if (!(pkg->device_desc->fss & FS_SAMPLES))
+  if (!json_reader_read_member (reader, PKG_TAG_SAMPLES))
     {
-      ret = 0;
-      goto cleanup_reader;
-    }
-
-  if (!json_reader_read_member (reader, PKG_TAG_SAMPLES)
-      || !json_reader_is_array (reader))
-    {
-      debug_print (1, "No samples found\n");
+      if (pkg->device_desc->filesystems & FS_SAMPLES)
+	{
+	  debug_print (1, "No samples found\n");
+	}
       control->parts = 1;	// Only payload and it's done.
       control->part = 0;
       set_job_control_progress (control, 1.0);
       goto cleanup_reader;
     }
 
+  if (!json_reader_is_array (reader))
+    {
+      error_print ("Member '%s' is not an array. Skipping samples...\n",
+		   PKG_TAG_SAMPLES);
+      ret = -1;
+      goto cleanup_reader;
+    }
+
   wave = g_byte_array_sized_new (zstat.size);
   raw = g_byte_array_sized_new (MAX_PACKAGE_LEN);
   elements = json_reader_count_elements (reader);
